package gallery
{
	import flash.display.Loader;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.TimerEvent;
	import flash.utils.Timer;
	
	import preloader.SimpleLinePreloader;
	
	import tools.BulkLoader;
	import tools.Statics;
	
	import transitions.TransitionManager;
	
	public class SlideShow extends Sprite
	{
		public static var itemAreaReady:String = "itemAreaReady"; 
		
		private var itemLoader:Loader;
		private var itemsPath:Array;
		private var itemTransitions:Array;
		private var itemSprite:Sprite;
		private var itemSpriteWidth:Number;
		private var itemSpriteHeight:Number;
		private var currentItem:IGalleryItem;
		private var nextItem:IGalleryItem;
		private var currentItemInd:Number;
		private var loadingItemInd:Number;
		private var timer:Timer;
		private var itempreLoader:SimpleLinePreloader;
		public function SlideShow(itemSpriteWidth:Number,itemSpriteHeight:Number,itemsPath:Array,itemTransitions:Array)
		{
			this.itemSpriteWidth = itemSpriteWidth;
			this.itemSpriteHeight = itemSpriteHeight;
			this.itemsPath = itemsPath;
			this.itemTransitions = itemTransitions;
			
			timer = new Timer(Statics.slideShowTime*1000,1);
			timer.addEventListener(TimerEvent.TIMER,showNextItem);
			
			TransitionManager.transitionAreaWidth = Statics.itemAreaWidth;
			TransitionManager.transitionAreaHeight = Statics.itemAreaHeight;
			
			
			var itemSpriteMask:Sprite = new Sprite();
			itemSpriteMask.graphics.lineStyle(0, 0, 0);
			itemSpriteMask.graphics.beginFill(0xFFFFFF, 1);
			itemSpriteMask.graphics.drawRect(0, 0, itemSpriteWidth, itemSpriteHeight);
			itemSpriteMask.graphics.endFill();
			addChild(itemSpriteMask);
			this.mask = itemSpriteMask;
			
			itemSprite = new Sprite();
			addChild(itemSprite);
			
			currentItem = new ImageItem("",itemSpriteWidth,itemSpriteHeight);
			itemSprite.addChild(currentItem as Sprite);
			nextItem = new ImageItem("",itemSpriteWidth,itemSpriteHeight);
			
			currentItemInd = -1;
			loadingItemInd = -1;
			
			itempreLoader = new SimpleLinePreloader(0xFF0000,0,0,itemSpriteWidth);
			addChild(itempreLoader);
			
			var shapeURLs:Array = new Array();
			
			for (var i:uint; i < itemTransitions.length; i++) {
				if ( itemTransitions[i] != null && itemTransitions[i].type == TransitionManager.Shapes) {
					if(itemTransitions[i].shapeURL!=""){
						shapeURLs.push(itemTransitions[i].shapeURL)
					}
				}
			}
			if (Statics.transition.type == TransitionManager.Shapes){
				if(Statics.transition.shapeURL!=null){
					shapeURLs.push(Statics.transition.shapeURL)
				}
			}
			
			Statics.shapeLoader = new BulkLoader(shapeURLs);
			Statics.shapeLoader.addEventListener("AllLoaded", shapesLoaded);
			Statics.shapeLoader.load();
			
			
		}
		
		private function shapesLoaded(e:Event):void{
			dispatchEvent(new Event(itemAreaReady));
		}
		
		public function loadItem(ind:Number):void
		{
			if (ind == currentItemInd || ind == loadingItemInd) {
				return;
			}
			if (nextItem && nextItem.isLoading()){
				nextItem.stopLoading();
			}
			loadingItemInd = ind;
			nextItem = new ImageItem(itemsPath[ind],itemSpriteWidth,itemSpriteHeight,itempreLoader);
			Sprite(nextItem).addEventListener("ItemReady",itemLoaded);
			nextItem.load();
		}
		
		private function itemLoaded(e:Event):void
		{
			Sprite(nextItem).removeEventListener("ItemReady",itemLoaded);
			TransitionManager.removeTransition();
			itemSprite.addChild(nextItem as Sprite);
			
			Sprite(nextItem).visible = false;
			currentItemInd = loadingItemInd;
			loadingItemInd = -1;
			if(!timer.running){
				showNewItem();
			}
		}
		private function transitionCompleted():void{
			itemSprite.removeChild(Sprite(currentItem));
			currentItem = nextItem;
			nextItem = null;
			
			loadItem((currentItemInd+1) % itemsPath.length);
			timer.reset();
			timer.start();
			//trace("before: "+System.totalMemory);
		}
		private function showNextItem(e:Event):void{
			if (loadingItemInd == -1){
				showNewItem();
			}
			//loadItem((currentItemInd+1) % itemsPath.length);
		}
		private function showNewItem():void {
			Sprite(nextItem).visible = true;
			if ( itemTransitions[currentItemInd] == null){
				TransitionManager.addTransition(Sprite(nextItem), Sprite(currentItem) , Statics.transition,transitionCompleted);
			}else{
				TransitionManager.addTransition(Sprite(nextItem), Sprite(currentItem) , itemTransitions[currentItemInd],transitionCompleted);
			}
		}
	}
}